/*
	FloatToIntCmd.cc

	Will Eighteen, 2000: will@opaline.demon.co.uk
*/                                                                             

#include "FloatToIntCmd.h"
#include <FileAccess.h>
#include "wave/waveclass.h"
#include <MessageDialogManager.h>


FloatToIntCmd::FloatToIntCmd ( char *name, int active, char *floatFile, char *intFile ) : BreakCmd ( name, active )
{
	strncpy ( _srcFile, floatFile, BUFSIZE );
	_srcFile [ sizeof ( _srcFile ) - 1 ] = 0;
	strncpy ( _dstFile, intFile, BUFSIZE );
	_dstFile [ sizeof ( _dstFile ) - 1 ] = 0;
	_once = 0;
	_percentDone = 0;
	_error = 0;
	_offset = 0;
	_scale = 1;
}


FloatToIntCmd::~FloatToIntCmd()
{	
	// empty
}
	

void FloatToIntCmd::doCommand()
{
	if ( _done == True )
		return;
	
	int percent;
	char text[5];
	int error = 0;

	if ( !_once )
	{
		// can we open source for read?
		FileAccess *src = new FileAccess ( _srcFile, "r" );
		
		if ( src->errNum() != 0 )
		{
			char msg[BUFSIZE*2];
			
			error++;
			snprintf ( msg, sizeof ( msg ), "%s\nFile read error: %s", _srcFile, src->error() );
			msg [ sizeof ( msg ) - 1 ] = 0;
			messageDialogManager->post ( msg );
		}
		
		delete src;
		
		// can we open dst for write?
		FileAccess *dst = new FileAccess ( _dstFile, "w" );
		
		if ( dst->errNum() != 0 )
		{
			char msg[BUFSIZE*2];
			
			error++;
			snprintf ( msg, sizeof ( msg ), "%s\nmFile write error: %s", _dstFile, dst->error() );
			msg [ sizeof ( msg ) - 1 ] = 0;
			messageDialogManager->post ( msg );
		}
		
		delete dst;
		
		if ( error )
		{
			_done = True;
			_error = error;
			return;
		}
		
		FileAccess *in = new FileAccess ( _srcFile, "r" );
		FileAccess *out = new FileAccess ( _dstFile, "w"  );
		
		_in = in;
		_out = out;

		_in->elementSize ( sizeof ( float ) );
		_in->numElements ( 1 );
		_out->elementSize ( sizeof ( int ) );
		_out->numElements ( 1 );

		updateTimeout ( ( int ) ( _in->fileSize() /	sizeof ( float ) ) + 1 );
		_once = 1;
	}
	
	if ( _in->filePos() < _in->fileSize() )
	{
		_in->readF ( &_fValue );
		_iValue = ( int ) ( ( _fValue + ( float ) _offset ) * ( float ) _scale );
		_out->writeF ( &_iValue );
		
		percent = ( int ) ( ( float ) _in->filePos() / ( float ) _in->fileSize() * 100 );

		if ( _percentDone != percent )
		{
			_percentDone = percent;
			snprintf ( text, 5, "%d%%", _percentDone );
			updateMessage ( text );
		}
	}
	else
	{
		delete _in;
		delete _out;
		_done = True;
	}
}

