/*
    TemplateInfoCmd.cc

    Will Eighteen: will@opaline.demon.co.uk
*/

#include "TemplateInfoCmd.h"
#include <DMenuCmd.h>
#include "OpResources.h"
#include "AppWindow.h"	// access to template loaded at top level
#include <InfoDialogManager.h>
#include <MessageDialogManager.h>


TemplateInfoCmd::TemplateInfoCmd ( const char *name, int active ) : NoUndoCmd ( name, active )
{
	_menuRealised = False;
}


void TemplateInfoCmd::doCommand()
{
	unsigned int n;
	unsigned int maxNameLength = 0;
	
	if ( templatePak->Count() == 0 )
		return;
		
	if ( _menuRealised == True )
		return;

	_arrayItems = ( unsigned int ) templatePak->Count();

	for ( n = 0; n < _arrayItems; n++ )
		if ( strlen ( (*templatePak)[n].name ) > maxNameLength )
			maxNameLength = strlen ( (*templatePak)[n].name );

	n = 0;
	maxNameLength++;
	_arrayItems++;
	
	char strings [ _arrayItems ] [ maxNameLength ];

	for ( n = 0; n < ( _arrayItems - 1 ); n++ )
		strcpy ( ( char * ) &strings [ n ], (*templatePak)[n].name );

	strcpy ( strings [ n ], "Quit" );

	_menu = new DMenuCmd ( "Template info menu", TRUE, FALSE,
								&TemplateInfoCmd::dMenuSelectionCallback,
								( void * ) this, "Current template files",
								_arrayItems, maxNameLength, ( char * ) &strings );

	_menuRealised = True;
	_menu->execute();
}


void TemplateInfoCmd::dMenuSelectionCallback ( void *clientData, unsigned int selectedItemNum, char *label )
{
	TemplateInfoCmd *obj = ( TemplateInfoCmd * ) clientData;				

	if ( strcmp ( label, "Quit" ) == 0 )
	{
		obj->_menuRealised = False;
		delete obj->_menu;
	}
	else
	{
		int count, found;
		
		// scan base list for returned name
		for ( count = 0; count < templatePak->Count(); count++ )
		{
			found = 0;
			
			if ( strcmp ( label, (*templatePak)[count].name ) == 0 )
			{
				obj->templateInfo ( (*templatePak)[count].templateNum );
				found = 1;
				count = templatePak->Count();
			}
		}
		
		if ( !found && mixwaveResources->enableMessagePopups() == True )
		{
			char msg[BUFSIZE];
			snprintf ( msg, sizeof ( msg ), "Error: %s\nFile not loaded.", label );
			msg [ sizeof ( msg ) - 1 ] = 0;
			messageDialogManager->post ( msg );
		}
	}
}


void TemplateInfoCmd::templateInfo ( unsigned int selectedItemNum )
{
	if ( selectedItemNum == 0 )
	{
		_menuRealised = False;
		return;
	}

	char buf[4096];
	char intText[20];

	_selectedItem = selectedItemNum;

	for ( int count = 0; count < templatePak->Count(); count++ )
		if ( (*templatePak)[count].infoNum == selectedItemNum )
			return;	// info is already displayed for this wave

	strncpy ( buf, (*templatePak)[selectedItemNum-1].name, sizeof ( buf ) );
	buf [ sizeof ( buf ) - 1 ] = 0;
	strncat ( buf, " ", sizeof ( buf ) - strlen ( buf ) - 1 );
     
	if ( (*templatePak)[selectedItemNum-1].dataSize == 0 ) 
		strncat ( buf, " initialised.\n", sizeof ( buf ) - strlen ( buf ) - 1 );	
	else
	{
		strncat  ( buf, "\nData size: ", sizeof ( buf ) - strlen ( buf ) - 1 );
		snprintf ( intText, sizeof ( intText ), "%ld", (*templatePak)[selectedItemNum-1].dataSize );
		intText [ sizeof ( intText ) - 1 ] = 0;
		strncat  ( buf, intText, sizeof ( buf ) - strlen ( buf ) - 1 );
		strncat  ( buf, " samples\n", sizeof ( buf ) - strlen ( buf ) - 1 );

		if ( strlen ( (*templatePak)[selectedItemNum-1].syncFile ) != 0 )
		{
			char temp[BUFSIZE];
			snprintf ( temp, sizeof ( temp ), "Linked to %s\nWave/Template data ratio %f\n",
						(*templatePak)[selectedItemNum-1].syncFile,
						(*templatePak)[selectedItemNum-1].syncRatio );
			temp [ sizeof ( temp ) - 1 ] = 0;
			strncat ( buf, temp, sizeof ( buf ) - strlen ( buf ) - 1);
		}

		if ( (*templatePak)[selectedItemNum-1].graphNum == 0 )	// or the templatePak selection isn't the current one in the viewer
		{
			if ( (*templatePak)[selectedItemNum-1].startSel != (*templatePak)[selectedItemNum-1].endSel )
			{
				strncat ( buf, "Selection on last view popdown:\n", sizeof ( buf ) - strlen ( buf ) - 1 );
				snprintf ( intText, sizeof ( intText ), "%ld", (*templatePak)[selectedItemNum-1].startSel );
				intText [ sizeof ( intText ) - 1 ] = 0;
				strncat ( buf, intText, sizeof ( buf ) - strlen ( buf ) - 1 );
				strncat ( buf, " to ", sizeof ( buf ) - strlen ( buf ) - 1 );
				snprintf ( intText, sizeof ( intText ), "%ld samples ", (*templatePak)[selectedItemNum-1].endSel );
				intText [ sizeof ( intText ) - 1 ] = 0;
				strncat ( buf, intText, sizeof ( buf ) - strlen ( buf ) - 1 );
			}

			if ( (*templatePak)[selectedItemNum-1].startRange != (*templatePak)[selectedItemNum-1].endRange )
			{
				snprintf ( intText, sizeof ( intText ), "(%ld loaded)\n", (*templatePak)[selectedItemNum-1].intArraySize );
				intText [ sizeof ( intText ) - 1 ] = 0;
				strncat ( buf, intText, sizeof ( buf ) - strlen ( buf ) - 1 );
			}
		}
     }

	if ( (*templatePak)[selectedItemNum-1].dataType == 0 )
		strncat  ( buf, "Data type: integer", sizeof ( buf ) - strlen ( buf ) - 1 );
	else
		strncat ( buf, "Data type: floating point", sizeof ( buf ) - strlen ( buf ) - 1 );

	(*templatePak)[selectedItemNum-1].infoNum = (*templatePak)[selectedItemNum-1].templateNum;

	infoDialogManager->post ( buf, ( void * ) this,
								&TemplateInfoCmd::infoOkCallback,
								&TemplateInfoCmd::infoCancelCallback );
}


void TemplateInfoCmd::infoOkCallback ( void *clientData )
{
	TemplateInfoCmd *obj = ( TemplateInfoCmd * ) clientData;

	// this is an unreliable guide to the original item, but ensures valid
	// selection although perhaps of the wrong item on the new list
	if ( templatePak->Count() < ( int ) ( obj->_arrayItems - 1 ) )	// deletion occurred
		obj->_selectedItem -= ( ( obj->_arrayItems - 1 ) - ( unsigned int ) templatePak->Count() );

	(*templatePak)[obj->_selectedItem - 1].infoNum = 0;	// re-enable info popup
}


void TemplateInfoCmd::infoCancelCallback ( void *clientData )
{
	TemplateInfoCmd *obj = ( TemplateInfoCmd * ) clientData;

	if ( templatePak->Count() < ( int ) ( obj->_arrayItems - 1 ) )	// deletion occurred
		obj->_selectedItem -= ( ( obj->_arrayItems - 1 ) - ( unsigned int ) templatePak->Count() );

	(*templatePak)[obj->_selectedItem - 1].infoNum = 0;	// re-enable info popup
} 
